
public class News : Gazette.Plugin.Base
{
	struct News {
		string title;
		string url;
	}
	public override Service service {get; protected set;}
	Settings settings;
	Soup.Session session;
	Soup.Message message;
	News[] news;
    construct {
        id = "news";
        name = _("News");
    }
	public News ()
	{
		settings = new Settings ("org.pantheon.gazette.news");
		settings.changed.connect( (key) => { if (key != "update-interval") update(); });
		session = new Soup.SessionAsync ();
	}
    public override Service create_service () {
        service = new Service(this);
		session = new Soup.SessionAsync ();
	    set_update_interval(settings.get_int("update-interval"));
        service.title = _("News");
        return service;
    }
	public override bool update ()
	{   debug("Updating News");
	    service.hide_reload_label ();
	    service.clear_contents ();
	    service.add_content(_("Loading news,\nplease wait."));
        var soup_url = settings.get_string ("rss-url");
		message = new Soup.Message ("GET", soup_url);
	    session.queue_message (message, (session, m) => {
		    var output = (string)m.response_body.data;
            if (output == null) {
                service.clear_contents ();
                service.show_reload_label ();
                return;
            }
		    var offset = output.index_of ("<item>");
		    news = new News[10];
		    for (var i = 0; i < 10; i++) {
			    offset = output.index_of ("<title>", offset) + 7;
			    var title = output.substring (offset, output.index_of ("</title>", offset) - offset);
			    offset = output.index_of ("<link>", offset) + 6;
			    var url = output.substring (offset, output.index_of ("</link>", offset) - offset);
			    news[i] = {ellipsize(title, 40), url};
		    }
		    service.clear_contents ();
		    for (var i = 0; i < news.length; i++) {
    		    service.add_content_with_link (news[i].title , news[i].url);
		    }
		});
		return true;
	}
	
	public override Gtk.Widget? create_config_widget () {
        var container = new Gtk.Grid ();
        container = new Gtk.Grid ();
        container.margin = 12;
        container.row_spacing = 12;
        container.column_spacing = 12;
        container.column_homogeneous = true;
        
        settings = new Settings ("org.pantheon.gazette.news");
        var rss_url = settings.get_string("rss-url");
        var update_interval = settings.get_int("update-interval");
        
        // URL LABEL
        var rssLabel = new Gtk.Label (_("URL") + " :");
        container.attach(rssLabel, 0, 0, 1, 1);
        rssLabel.set_alignment (1, 0);
        
        // URL ENTRY
        var rssEntry = new Gtk.Entry ();
        rssEntry.set_text (rss_url);
        container.attach(rssEntry, 1, 0, 2, 1);

        rssEntry.changed.connect (() => {

        settings.set_string("rss-url", rssEntry.get_text ());
        });

        // REFRESH LABEL
        var refreshLabel = new Gtk.Label (_("Refresh Interval") + " :");
        container.attach(refreshLabel, 0, 1, 1, 1);
        refreshLabel.set_alignment (1, 0);
        
        // REFRESH ENTRY
        var scale = new Gtk.Scale.with_range (Gtk.Orientation.HORIZONTAL, 0, 3600000, 60000);
        scale.set_draw_value (false);
        scale.add_mark (300000, Gtk.PositionType.BOTTOM, _("5 min"));
        scale.add_mark (1200000, Gtk.PositionType.BOTTOM, _("20 min"));
        scale.add_mark (2400000, Gtk.PositionType.BOTTOM, _("40 min"));
        scale.add_mark (3600000, Gtk.PositionType.BOTTOM, _("1 hour"));

        scale.set_value (update_interval);

        scale.value_changed.connect (() => {
            settings.set_int("update-interval", (int) scale.get_value() );
        });

        container.attach(scale, 1, 1, 2, 1);
        
        // RESET BUTTON
        var resetButton = new Gtk.Button.with_label ( _("Reset") );
        container.attach(resetButton, 2, 2, 1, 1);
        resetButton.clicked.connect(() => {
            settings.reset ("rss-url");
            settings.reset ("update-interval");
            rss_url = settings.get_string("rss-url");
            update_interval = settings.get_int("update-interval");
            rssEntry.set_text (rss_url);
            scale.set_value (update_interval);
        });
        return container;
    }
}

public News plugin_init () {
    // types are registered automatically
    return new News ();
}
