
const string [] condition_codes = {
	"c", // tornado
	"0", // tropical storm
	"c", // hurricane
	"6", // severe thunderstorms
	"6", // thunderstorms
	"2", // mixed rain and snow
	"3", // mixed rain and sleet
	"2", // mixed snow and sleet
	"e", // freezing drizzle
	"3", // drizzle
	"7", // freezing rain
	"3", // showers
	"3", // showers
	"6", // snow flurries
	"G", // light snow showers G
	"#", // blowing snow #
	"h", // snow "
	"7", // hail
	"7", // sleet
	"F", // dust
	"F", // foggy
	"F", // haze
	"M", // smoky M
	"c", // blustery
	"S", // windy S
	"h", // cold
	"N", // cloudy N
	"4", // mostly cloudy (night) 4
	"3", // mostly cloudy (day) 3
	"K", // partly cloudy (night) K
	"J", // pardetly cloudy (day) J
	"C", // clear (night) C
	"B", // sunny B
	"2", // fair (night) 2
	"B", // fair (day)  B
	"X", // mixed rain and hail X
	"1", // hot 1
	"1", // isolated thunderstorms
	"1", // scattered thunderstorms
	"1", // scattered thunderstorms
	"e", // scattered showers
	"#", // heavy snow #
	"2", // scattered snow showers
	"#", // heavy snow #
	"H", // partly cloudy H
	"1", // thundershowers
	"2", // snow showers
	"1", // isolated thundershowers
	")" // not available )
};

public class Weather : Gazette.Plugin.Base {
    string [] condition_texts = {
        	_("Tornado"), // tornado
	        _("Tropical storm"), // tropical storm
	        _("Hurricane"), // hurricane
	        _("Severe thunderstorms"), // severe thunderstorms
	        _("Thunderstorms"), // thunderstorms
	        _("Mixed rain and snow"), // mixed rain and snow
	        _("Mixed rain and sleet"), // mixed rain and sleet
	        _("Mixed snow and sleet"), // mixed snow and sleet
	        _("Freezing drizzle"), // freezing drizzle
	        _("Drizzle"), // drizzle
	        _("Freezing rain"), // freezing rain
	        _("Showers"), // showers
	        _("Showers"), // showers
	        _("Snow flurries"), // snow flurries
	        _("Light snow showers"), // light snow showers G
	        _("Blowing snow"), // blowing snow #
	        _("Snow"), // snow "
	        _("Hail"), // hail
	        _("Sleet"), // sleet
	        _("Dust"), // dust
	        _("Foggy"), // foggy
	        _("Haze"), // haze
	        _("Smoky"), // smoky M
	        _("Blustery"), // blustery
	        _("Windy"), // windy S
	        _("Cold"), // cold
	        _("Cloudy"), // cloudy N
	        _("Mostly cloudy"), // mostly cloudy (night) 4
	        _("Mostly cloudy"), // mostly cloudy (day) 3
	        _("Partly cloudy"), // partly cloudy (night) K
	        _("Partly cloudy"), // partly cloudy (day) J
	        _("Clear"), // clear (night) C
	        _("Sunny"), // sunny B
	        _("Fair"), // fair (night) 2
	        _("Fair"), // fair (day)  B
	        _("Mixed rain and hail"), // mixed rain and hail X
	        _("Hot"), // hot 1
	        _("Isolated thunderstorms"), // isolated thunderstorms
	        _("Scattered thunderstorms"), // scattered thunderstorms
	        _("Scattered thunderstorms"), // scattered thunderstorms
	        _("Scattered showers"), // scattered showers
	        _("Heavy snow"), // heavy snow #
	        _("Scattered snow showers"), // scattered snow showers
	        _("Heavy snow"), // heavy snow #
	        _("Partly cloudy"), // partly cloudy H
	        _("Thundershowers"), // thundershowers
	        _("Snow showers"), // snow showers
	        _("Isolated thundershowers"), // isolated thundershowers
	        _("Not available") // not available )
    };



	string unit;
	string text;
    Soup.SessionAsync session;
    Soup.Message message;
	Settings settings;
    construct {
        settings = new Settings ("org.pantheon.gazette.weather");
        id = "weather";
        name = _("Weather");
    }
    public override Service service {get; protected set;}

	public override Service create_service() {
	    service = new Service (this);
		settings.changed.connect( (key) => { update(); });
		session = new Soup.SessionAsync ();
	    Timeout.add(settings.get_int("update-interval"), update);
	    return service;
	}

	private Gee.HashMap<string,string> get_attributes (string data, string tagname, string [] attrs, int offset = 0) {
		var start = data.index_of ("<" + tagname, offset) + tagname.length + 1;
		var end = data.index_of ("/>", start);
		var tmp_data = data.substring (start, end - start);

		var res = new Gee.HashMap<string,string> ();
		foreach (var attr in attrs) {
				res[attr] = get_attribute_value (tmp_data, attr);
		}
		return res;
	}

	private string get_attribute_value (string data, string attr) {
		var start = data.index_of (attr + "=\"") + attr.length + 2;
		var end = data.index_of ("\"", start);
		return data.substring (start, end - start);
	}

	public override bool update() {
	    debug ("Updating Weather");
	    string woeid;
	    service.clear_contents ();
	    service.hide_reload_label ();
	    service.add_content (_("Loading weather,\nplease wait."));
        woeid = settings.get_int ("weather-id").to_string ();
		unit = settings.get_string ("weather-unit") == "Fahrenheit" ? "f" : "c";
		var url = "http://weather.yahooapis.com/forecastrss";
		url += "?u=" + unit;
		url += "&w=" + woeid;
		debug(url);
		message = new Soup.Message ("GET", url);
	    session.queue_message(message, (session, m) => {
            var data = (string)message.response_body.data;
            if (data == null) {
                service.clear_contents ();
                service.show_reload_label ();
                return;
            }

			var current = get_attributes (data, "yweather:condition", {"temp", "code"});
			var forecast = get_attributes (data, "yweather:forecast", {"day", "date", "low", "high", "code"},
				data.index_of ("<yweather:forecast"));
			var forecast2 = get_attributes (data, "yweather:forecast", {"day", "date", "low", "high", "code"},
				data.index_of ("<yweather:forecast") + 10);

			var today = new DateTime.now_local ().format (_("%A"));

			text =
                @"<span face='Open Sans Light' font='24'>$today</span>" +
                "<span face='Open Sans Light' font='16'> // <i>" + condition_texts[int.parse (current["code"])] +"</i></span>\n" +
				"<span face='gazetteweather' font='68'>" + condition_codes[int.parse (current["code"])] + "</span>" +
				"<span face='Raleway' weight='100' font='72'> " + current["temp"] + "</span>" +
				"<span face='Raleway' weight='100' font='40'> ° " + unit.up () + "</span>\n" + 
				"<span face='gazetteweather' font='30'>" + condition_codes[int.parse (forecast["code"])] + "</span>" +
				"<span face='Open Sans Light' font='26'> " + forecast["day"] + "     </span>" +
				"<span face='gazetteweather' font='30'>" + condition_codes[int.parse (forecast2["code"])] + "</span>" +
				"<span face='Open Sans Light' font='26'> " + forecast2["day"] + "</span>\n"+

				"<span face='Raleway' font='21'>" + forecast["low"] + " - " + forecast["high"] +"°"+unit.up()+"      </span>"+
				"<span face='Raleway' font='21'>" + forecast2["low"] + " - " + forecast2["high"] +"°"+unit.up()+"      </span>";
			service.clear_contents ();
			service.add_content_marked_up(text);
	    });
	    return true;
	}
	
    public override Gtk.Widget? create_config_widget () {
        // VAR INIT
        var container = new Gtk.Grid ();
        container = new Gtk.Grid ();
        container.margin = 12;
        container.row_spacing = 12;
        container.column_spacing = 12;
        container.column_homogeneous = true;
        var weather_id = settings.get_int("weather-id");

        var weather_unit = settings.get_string("weather-unit");

        // ID LABEL
        var weatherIdLabel = new Gtk.Label ("* "+ _("Location ID") + " / " +  _("Unit") + " :");
        container.attach(weatherIdLabel, 0, 0, 1, 1);

        // ID ENTRY
        var weatherIdEntry = new Gtk.SpinButton.with_range (10000, 99999999, 1);
        weatherIdEntry.set_value (weather_id);
        weatherIdEntry.value_changed.connect (() => {
            settings.set_int("weather-id", weatherIdEntry.get_value_as_int ());
        });
        container.attach(weatherIdEntry, 1, 0, 1, 1);

        // ID HELP MESSAGE
        var weatherIdHelp = new Gtk.Label ("<span font_desc=\"8.0\">*" + _("Example: If the URL is <i>http://weather.yahoo.com/united-kingdom/england/london-44418/</i> the city's ID is 44418.") + "</span>");
        weatherIdHelp.set_use_markup (true);
        weatherIdHelp.set_line_wrap (true);
        container.attach(weatherIdHelp, 1, 1, 2, 1);
        weatherIdHelp.set_alignment (0, 0);

        // UNIT ENTRY
        var weatherUnitBox = new Gtk.ComboBoxText ();
        string new_unit = "";
        weatherUnitBox.append_text ("Celsius");
        weatherUnitBox.append_text ("Fahrenheit");
        if (weather_unit == "'Celsius'")
            weatherUnitBox.active = 1;
        else
            weatherUnitBox.active = 0;
        container.attach(weatherUnitBox, 2, 0, 1, 1);
        weatherUnitBox.changed.connect (() => {

                if (weatherUnitBox.get_active_text() == "Celsius"){
                    new_unit = "Celsius";
                }
                else{
                    new_unit = "Fahrenheit";
                }
                settings.set_string("weather-unit", new_unit);
        });

       // UNIT LABEL [F/C]
        var weatherUnitLabel = new Gtk.Label ( "<a href='http://weather.yahoo.com'>" + _("Search Location ID on Yahoo!® Weather") + "</a> :");
        container.attach(weatherUnitLabel, 0, 1, 1, 1);
        weatherUnitLabel.set_alignment (1, 0);
        weatherUnitLabel.set_use_markup (true);
        weatherUnitLabel.set_line_wrap (true);

        // RESET BUTTON
        var resetButton = new Gtk.Button.with_label ( _("Reset") );
        container.attach(resetButton, 2, 2, 1, 1);
        resetButton.clicked.connect(() => {
            settings.reset("weather-unit");
            settings.reset("weather-id");
            weather_id = settings.get_int("weather-id");
            weather_unit = settings.get_string("weather-unit");
            weatherIdEntry.set_value (weather_id);
            if (weather_unit == "'Celsius'")
                weatherUnitBox.active = 1;
            else
                weatherUnitBox.active = 0;
        });
        return container;
    }
}

public Weather plugin_init () {
    // types are registered automatically
    return new Weather ();
}
