using Clutter;

public class GlobalSettings : Granite.Services.Settings
{
	public string[] service_states { get; set; }
    public string[] enabled_services { get; set; }
    public bool show_welcome { get; set; }
	static GlobalSettings? instance;

	public GlobalSettings ()
	{
		base ("org.pantheon.gazette");
	}

	public static GlobalSettings get_default ()
	{
		if (instance == null)
			instance = new GlobalSettings ();

		return instance;
	}
}

public class Gazette.App : Granite.Application
{
    public static PluginManager plugin_manager;

	construct
	{
		program_name = "Gazette";
		application_id = "org.pantheon.gazette";
	}
	const int MARGIN = 50;
//    public PluginManager plugin_manager;
    private Gee.HashMap<string,string> states;
    private GazetteWindow window;

	public override void activate () {
	    plugin_manager = new PluginManager(new GLib.Settings("org.pantheon.gazette"), "enabled-services", Constants.PLUGINDIR);
	    plugin_manager.plugin_enabled.connect(load_plugin);
		if (get_windows ().length () > 0)
			return;
		window = new GazetteWindow ();
		window.set_application (this);
		var size = window.reposition ();
		window.show_all ();
		var timeout = 0;

		if (GlobalSettings.get_default ().show_welcome) {
    		WelcomeMessage.show_on_window(window);
    		timeout = 2000;
		}

        Timeout.add(timeout, () => {
            plugin_manager.load_plugins ();
            var plugins = plugin_manager.get_enabled_plugins ();
            states = new Gee.HashMap<string,string>();
            foreach (string s in GlobalSettings.get_default ().service_states) {
                debug (s);
                var arr = s.split(":");
                states[arr[0]] = arr[1];
            }
            foreach (var plugin in plugins) {
                load_plugin (plugin);
            }
            Process.signal(ProcessSignal.TERM, save_states);
            Process.signal(ProcessSignal.INT, save_states);
            return false;
        });
	}
    public void load_plugin (Plugin.Base plugin) {
        debug ("Loading service");
        window.stage.add_child (plugin.create_service ());
        if (states.has_key(plugin.id)) {
            plugin.service.settings_deserialize(states[plugin.id]);
        }
        Idle.add(() => { plugin.update(); return false;});
    }
    public static void save_states () {
        debug("Saving states before exiting");
        string[] plugin_states = {};
        var plugins = Gazette.App.plugin_manager.get_enabled_plugins();
	    foreach (var p in plugins) {
	        plugin_states += p.id.to_string () + ":" + p.service.settings_serialize();
	        p.destroy ();
        }
        GlobalSettings.get_default ().service_states = plugin_states;
        Process.exit(0);
    }
}


int main (string [] args)
{
	GtkClutter.init (ref args);
	return new Gazette.App ().run(args);
}


